package main;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Class responsible for generating combinations using the Each-choice data
 * combinations criteria. The criteria generates a set of combinations ensuring
 * that each value for each parameter appears in at least one test case.
 * 
 * @author ernestocid
 * 
 */
public class EachChoice extends Criteria {

	
	public EachChoice(List<List<String>> parametersInputValues) {
		super(parametersInputValues);
	}

	

	/**
	 * Create the combinations according to the criteria. The method combines
	 * values of the same index, when the list reaches its limit, it will always use
	 * its last values.
	 * 
	 * @return combinations: Set of combinations where each element is a combinations
	 * 							organized as a list;
	 */
	@Override
	public Set<List<String>> getCombinations() {
		if (getParametersInputValues().size() < 2) {
			return generateOneParameterCombination();
		} else {
			return generateCombinations();			
		}
	}



	private Set<List<String>> generateOneParameterCombination() {
		Set<List<String>> combinations = new HashSet<List<String>>();
		
		if(getParametersInputValues().size() == 1) {
			List<String> singleElement = new ArrayList<String>();
			singleElement.add(getParametersInputValues().get(0).get(0));
			combinations.add(singleElement);
			return combinations;
		} else {
			return combinations;
		}
	}



	private Set<List<String>> generateCombinations() {
		Set<List<String>> combinations = new HashSet<List<String>>();
		
		int maxNumberOfValues = getMaxNumberOfValues(getParametersInputValues());

		for (int blockIndex = 0; blockIndex < maxNumberOfValues; blockIndex++) {
			List<String> combination = new ArrayList<String>();

			for (List<String> parameterBlocks : getParametersInputValues()) {
				int numberOfBlocks = parameterBlocks.size();
				if (blockIndex < numberOfBlocks) {
					combination.add(parameterBlocks.get(blockIndex));
				} else {
					combination.add(getLastBlock(parameterBlocks));
				}
			}

			combinations.add(combination);
		}
		
		return combinations;
	}

	
	
	/**
	 * Given the possible values for a list of parameters, it returns the number
	 * of values of the parameter with more values.
	 * 
	 * @param parameters: Lists of parameter values;
	 * @return maxNumberofValues: the number of values of the parameter with
	 *         more values;
	 */
	private int getMaxNumberOfValues(List<List<String>> parameters) {
		int maxNumberOfValues = 0;

		for (List<String> parameterInputValues : parameters) {
			int numberOfValues = parameterInputValues.size();
			if (numberOfValues > maxNumberOfValues) {
				maxNumberOfValues = numberOfValues;
			}
		}

		return maxNumberOfValues;
	}
	
	

	private String getLastBlock(List<String> parameter) {
		return parameter.get(parameter.size() - 1);
	}
}